"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const pm2 = require("pm2");
const handlers = {};
const instanceId = Number(process.env.pm_id);
const instanceName = process.env.name;
/**
 * Message listener
 */
process.on('message', (packet) => __awaiter(this, void 0, void 0, function* () {
    if (typeof handlers[packet.topic] === 'function' && process.send) {
        const data = yield handlers[packet.topic](packet);
        process.send({ type: `process:${packet.data.targetInstanceId}`, data });
    }
}));
/**
 * Retrieves messages from sibling instances.
 */
exports.getMessages = function getMessages(topic, timeout = 1000) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            assert.equal(typeof handlers[topic], 'function');
            const packet = { topic, data: { targetInstanceId: instanceId } };
            const request = {
                pending: 0,
                responses: [yield handlers[topic](packet)],
                timeout: setTimeout(() => reject(new Error(`${topic} timed out`)), timeout),
            };
            const done = function done(err) {
                clearTimeout(request.timeout);
                if (err)
                    reject(err);
                else
                    resolve(request.responses);
            };
            pm2.connect(false, (err) => {
                if (err)
                    return done(err);
                pm2.list((err, processes) => {
                    if (err)
                        return done(err);
                    const instances = processes.filter((process) => process.name === instanceName && process.pm_id !== instanceId);
                    if (instances.length)
                        request.pending += instances.length;
                    else
                        return done();
                    const listener = new Promise((resolve, reject) => {
                        pm2.launchBus((err, bus) => {
                            if (err)
                                return reject(err);
                            bus.on(`process:${instanceId}`, (packet) => {
                                request.responses.push(packet.data);
                                request.pending -= 1;
                                if (!request.pending)
                                    resolve();
                            });
                        });
                    });
                    instances.forEach((instance) => {
                        if (typeof instance.pm_id === 'number') {
                            pm2.sendDataToProcessId(instance.pm_id, packet, (err) => err && done(err));
                        }
                    });
                    listener.then(() => done(), done);
                });
            });
        })).finally(() => pm2.disconnect());
    });
};
/**
 * Adds a message handler.
 */
exports.onMessage = function onMessage(topic, handler) {
    handlers[topic] = handler;
};
exports.default = { getMessages: exports.getMessages, onMessage: exports.onMessage };
//# sourceMappingURL=index.js.map